package net.stoerr.sudoku;

import java.util.ArrayList;
import java.util.List;


/**
 * A constraint over several points.
 * @author hps
 *
 * @param <P> type of the points
 */
public abstract class AbstractConstraint<P extends Comparable<P>> {
    protected final ConstraintStore<P> store;
    private final ArrayList<P> points;

    /**
     * Initialization - remembers the constraint store for future reference.
     * @param store
     */
    public AbstractConstraint(ConstraintStore<P> store) {
        this.store = store;
        this.points = new ArrayList<P>();
    }

    /**
     * Adds a point to the constraint.
     * @param idx
     */
    public void addPoint(P idx) {
        points.add(idx);
    }

    /**
     * Returns the list of points of the constraint.
     * @return list that should not be modified.
     */
    public List<P> getPoints() {
        return points;
    }

    /**
     * Notify constraint of a change and derive further changes.
     * @param idx
     * @param val
     */
    public abstract void propagate(P idx, String val);
}
