/**
 *
 */
package net.stoerr.sudoku;

import java.util.HashMap;
import java.util.Map;


/**
 * A constraint that ensures that all digits occur exactly once in its
 * domain.
 * @author hps
 * @param <P>
 *
 */
public class PermutationConstraint<P extends Comparable<P>>
    extends AbstractConstraint<P> {
    public PermutationConstraint(ConstraintStore<P> store) {
        super(store);
    }

    /**
     * Sets a digit into a field and removes its possibility from the other fields. Also
     * calls {@link #checkSoloConstraints()}.
     * @see net.stoerr.sudoku.AbstractConstraint#propagate(java.lang.Comparable, java.lang.String)
     */
    @Override
    public void propagate(P idx, String val) {
        if ((null != val) && (1 == val.length())) {
            for (P p : getPoints()) {
                if (!idx.equals(p)) {
                    String pval = store.getConstraint(p);
                    String nval = pval.replaceAll(val, "");

                    if (!pval.equals(nval)) {
                        store.setConstraint(p, nval);
                    }
                }
            }
        }

        checkSoloConstraints();
    }

    /**
     * Checks for digits that occur only once - then sets this field is occupied with this digit.
     */
    private void checkSoloConstraints() {
        Map<Character, Integer> charCount = new HashMap<Character, Integer>();

        // counts the chars in the domain
        for (P p : getPoints()) {
            String pval = store.getConstraint(p);

            for (char c : pval.toCharArray()) {
                Integer count = charCount.get(c);

                if (null == count) {
                    count = 0;
                }

                count += 1;
                charCount.put(c, count);
            }
        }

        // if digits occur once, set them.
        for (Map.Entry<Character, Integer> entry : charCount.entrySet()) {
            if (1 == entry.getValue()) {
                char c = entry.getKey();
                String cS = String.valueOf(c);

                for (P p : getPoints()) {
                    String pval = store.getConstraint(p);

                    if ((0 <= pval.indexOf(cS)) && !cS.equals(pval)) {
                        store.setConstraint(p, cS);
                    }
                }
            }
        }
    }
}
