package net.stoerr.sudoku;

import java.awt.BorderLayout;
import java.awt.FlowLayout;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTable;
import javax.swing.JTextPane;
import javax.swing.SwingConstants;

/**
 * This code was edited or generated using CloudGarden's Jigloo
 * SWT/Swing GUI Builder, which is free for non-commercial
 * use. If Jigloo is being used commercially (ie, by a corporation,
 * company or business for any purpose whatever) then you
 * should purchase a license for each developer using Jigloo.
 * Please visit www.cloudgarden.com for details.
 * Use of Jigloo implies acceptance of these licensing terms.
 * A COMMERCIAL LICENSE HAS NOT BEEN PURCHASED FOR
 * THIS MACHINE, SO JIGLOO OR THIS CODE CANNOT BE USED
 * LEGALLY FOR ANY CORPORATE OR COMMERCIAL PURPOSE.
 */
/**
 * Panel displaying the sudoku solver. (Visual editor.)
 */
public class SudokuPanel extends JPanel {
    private static final long serialVersionUID = 1L;
    private static final String HELPTEXT = "This applet is able to solve a 9x9 \"standard\" sudoku as you type in the given numbers. In each field all numbers that are currently possible are shown, after taking into account all the numbers that you have typed.\n"
            + "\n"
            + "Initially in every field all digits are possible. Thus, 123456789 is shown. Now click into some fields and type a number. The possible numbers in the other fields are changed according to the rules of sudoku. For example, 1349 means that in this field there can only be a 1, 3, 4 or 9 according to the numbers you typed in. If there are no numbers left in a field, you've typed a wrong number: there is a contradiction in the numbers you typed. But don't worry: you can always press \"Undo\" to retrace your steps.\n"
            + "\n"
            + "If you typed all numbers of the sudoku and there are still fields where several numbers are possible, the simple constraint propagation algorithm in the applet is not able to solve the sudoku as it is - though I have yet to find a sudoku where this simplest of all algorithms does not suffice. In that case just press \"Solve\" for executing a search that will find one solution - if there is one.\n"
            + "\n" + "Have fun!\n" + "\n" + "Hans-Peter Störr, www.stoerr.net, 2008 / 2009";
    private JPanel buttonPanel = null;
    private JLabel headline = null;
    private JTextPane helpTextPane;
    private JTable table = null;
    private JButton resetButton = null;
    private JButton undoButton = null;
    private SudokuModel model;
    private JButton solveButton = null;

    /**
     * This is the default constructor
     */
    public SudokuPanel() {
        super();
        initialize();
    }

    /**
     * This method initializes this
     */
    private void initialize() {
        setLayout(new BoxLayout(this, BoxLayout.Y_AXIS));
        {
            headline = new JLabel();
            this.add(headline);
            final BorderLayout headlineLayout = new BorderLayout();
            headline.setLayout(headlineLayout);
            headline.setText("Sudoku Solver");
            headline.setHorizontalAlignment(SwingConstants.CENTER);
        }
        {
            helpTextPane = new JTextPane();
            this.add(helpTextPane);
            helpTextPane.setText(HELPTEXT);
            helpTextPane.setEditable(false);
        }
        this.add(getTable());
        this.add(getButtonPanel());
    }

    /**
     * This method initializes buttonPanel
     * @return javax.swing.JPanel
     */
    private JPanel getButtonPanel() {
        if (buttonPanel == null) {
            buttonPanel = new JPanel();
            buttonPanel.setLayout(new FlowLayout());
            buttonPanel.add(getResetButton(), null);
            buttonPanel.add(getUndoButton(), null);
            buttonPanel.add(getSolveButton(), null);
        }

        return buttonPanel;
    }

    /**
     * This method initializes table
     * @return javax.swing.JTable
     */
    private JTable getTable() {
        if (table == null) {
            table = new JTable();
            model = sudokuModelFactory();
            model.reset();
            table.setModel(model);
            table.addKeyListener(new java.awt.event.KeyAdapter() {
                @Override
                public void keyTyped(final java.awt.event.KeyEvent e) {
                    int col = table.getEditingColumn();
                    int row = table.getEditingRow();

                    if (0 > col) {
                        col = table.getSelectedColumn();
                        row = table.getSelectedRow();
                    }

                    model.setValueAt(e.getKeyChar(), row, col);
                    table.repaint();
                }
            });
            table.setDefaultRenderer(Object.class, new SudokuCellRenderer());
            table.setShowGrid(false);
            table.setRowHeight(60);
            table.setCellSelectionEnabled(true);
        }

        return table;
    }

    /**
     * Factory for the used sudoku model.
     */
    protected SudokuModel sudokuModelFactory() {
        return new SudokuConstraintPropagator();
    }

    /**
     * This method initializes resetButton
     * @return javax.swing.JButton
     */
    private JButton getResetButton() {
        if (resetButton == null) {
            resetButton = new JButton();
            resetButton.setText("Reset");
            resetButton.setToolTipText("Reset all values");
            resetButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent e) {
                    model.reset();
                    table.repaint();
                }
            });
        }

        return resetButton;
    }

    /**
     * This method initializes undoButton
     * @return javax.swing.JButton
     */
    private JButton getUndoButton() {
        if (undoButton == null) {
            undoButton = new JButton();
            undoButton.setText("Undo");
            undoButton.setToolTipText("Undo last typed number");
            undoButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent e) {
                    model.undo();
                    solveButton.setEnabled(true);
                    table.repaint();
                }
            });
        }

        return undoButton;
    }

    /**
     * This method initializes solveButton
     * @return javax.swing.JButton
     */
    private JButton getSolveButton() {
        if (solveButton == null) {
            solveButton = new JButton();
            solveButton.setText("Solve");
            solveButton.setToolTipText("Find a solution that satisfies the currently typed numbers. The button is disabled if there is no solution - Undo to reenable.");
            solveButton.addActionListener(new java.awt.event.ActionListener() {
                public void actionPerformed(final java.awt.event.ActionEvent e) {
                    final boolean solved = model.solve();
                    solveButton.setEnabled(solved);
                    table.repaint();
                }
            });
        }

        return solveButton;
    }
}
